@@ -1,16 +1,6 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2012-2019 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 from __future__ import annotations
 
 """Representations specific to the CompCert C compiler family."""
@@ -20,8 +10,7 @@ import re
 import typing as T
 
 if T.TYPE_CHECKING:
-    from envconfig import MachineInfo
-    from ...environment import Environment
+    from ...envconfig import MachineInfo
     from ...compilers.compilers import Compiler
 else:
     # This is a bit clever, for mypy we pretend that these mixins descend from
@@ -30,16 +19,7 @@ else:
     # do). This gives up DRYer type checking, with no runtime impact
     Compiler = object
 
-ccomp_buildtype_args = {
-    'plain': [''],
-    'debug': ['-O0', '-g'],
-    'debugoptimized': ['-O0', '-g'],
-    'release': ['-O3'],
-    'minsize': ['-Os'],
-    'custom': ['-Obranchless'],
-}  # type: T.Dict[str, T.List[str]]
-
-ccomp_optimization_args = {
+ccomp_optimization_args: T.Dict[str, T.List[str]] = {
     'plain': [],
     '0': ['-O0'],
     'g': ['-O0'],
@@ -47,19 +27,19 @@ ccomp_optimization_args = {
     '2': ['-O2'],
     '3': ['-O3'],
     's': ['-Os']
-}  # type: T.Dict[str, T.List[str]]
+}
 
-ccomp_debug_args = {
+ccomp_debug_args: T.Dict[bool, T.List[str]] = {
     False: [],
-    True: ['-g']
-}  # type: T.Dict[bool, T.List[str]]
+    True: ['-O0', '-g']
+}
 
 # As of CompCert 20.04, these arguments should be passed to the underlying gcc linker (via -WUl,<arg>)
 # There are probably (many) more, but these are those used by picolibc
-ccomp_args_to_wul = [
+ccomp_args_to_wul: T.List[str] = [
         r"^-ffreestanding$",
         r"^-r$"
-] # type: T.List[str]
+]
 
 class CompCertCompiler(Compiler):
 
@@ -68,12 +48,14 @@ class CompCertCompiler(Compiler):
     def __init__(self) -> None:
         # Assembly
         self.can_compile_suffixes.add('s')
-        default_warn_args = []  # type: T.List[str]
-        self.warn_args = {'0': [],
-                          '1': default_warn_args,
-                          '2': default_warn_args + [],
-                          '3': default_warn_args + [],
-                          'everything': default_warn_args + []}  # type: T.Dict[str, T.List[str]]
+        self.can_compile_suffixes.add('sx')
+        default_warn_args: T.List[str] = []
+        self.warn_args: T.Dict[str, T.List[str]] = {
+            '0': [],
+            '1': default_warn_args,
+            '2': default_warn_args + [],
+            '3': default_warn_args + [],
+            'everything': default_warn_args + []}
 
     def get_always_args(self) -> T.List[str]:
         return []
@@ -82,9 +64,6 @@ class CompCertCompiler(Compiler):
         # As of now, CompCert does not support PIC
         return []
 
-    def get_buildtype_args(self, buildtype: str) -> T.List[str]:
-        return ccomp_buildtype_args[buildtype]
-
     def get_pch_suffix(self) -> str:
         return 'pch'
 
@@ -94,7 +73,7 @@ class CompCertCompiler(Compiler):
     @classmethod
     def _unix_args_to_native(cls, args: T.List[str], info: MachineInfo) -> T.List[str]:
         "Always returns a copy that can be independently mutated"
-        patched_args = []  # type: T.List[str]
+        patched_args: T.List[str] = []
         for arg in args:
             added = 0
             for ptrn in ccomp_args_to_wul:
@@ -105,7 +84,7 @@ class CompCertCompiler(Compiler):
                 patched_args.append(arg)
         return patched_args
 
-    def thread_flags(self, env: 'Environment') -> T.List[str]:
+    def thread_flags(self) -> T.List[str]:
         return []
 
     def get_preprocess_only_args(self) -> T.List[str]:
