@@ -2,6 +2,6 @@
 
 class Foo;
 
-int main(int argc, char **argv) {
+int main(void) {
     return 0;
 }
